import logging
from ..static import LOGGER_PRINT_LEVEL
logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(LOGGER_PRINT_LEVEL)

import pickle
import os
import collections
import sys

from .pycocoevalcap.bleu.bleu import Bleu
from .pycocoevalcap.rouge.rouge import Rouge
from .pycocoevalcap.meteor.meteor import Meteor

class Evaluate(object):
    def __init__(self):
        self.scorers = [
            (Bleu(4), ["Bleu_1", "Bleu_2", "Bleu_3", "Bleu_4"]),
            (Meteor(), "METEOR"),
            (Rouge(), "ROUGE_L")
        ]#,        (Cider(), "CIDEr")

    def convert(self, data):
        if isinstance(data, basestring):
            return data.encode('utf-8')
        elif isinstance(data, collections.Mapping):
            return dict(map(convert, data.items()))
        elif isinstance(data, collections.Iterable):
            return type(data)(map(convert, data))
        else:
            return data

    def score(self, ref, hypo):
        final_scores = {}
        for scorer, method in self.scorers:
            score, scores = scorer.compute_score(ref, hypo)
            if type(score) == list:
                for m, s in zip(method, score):
                    final_scores[m] = s
            else:
                final_scores[method] = score

        return final_scores

    def evaluate(self, get_scores=True, live=False, **kwargs):
        if live:
            temp_ref = kwargs.pop('ref', {})
            cand = kwargs.pop('cand', {})
        else:
            reference_path = kwargs.pop('ref', '')
            candidate_path = kwargs.pop('cand', '')

            # load caption data
            with open(reference_path, 'rb') as f:
                temp_ref = pickle.load(f)
            with open(candidate_path, 'rb') as f:
                cand = pickle.load(f)

        # make dictionary
        hypo = {}
        ref = {}
        i = 0
        for vid, caption in cand.items():
            hypo[i] = [caption]
            ref[i] = temp_ref[vid]
            i += 1

        # compute scores
        final_scores = self.score(ref, hypo)
        #"""
        # print out scores
        print ('Bleu_1:\t', final_scores['Bleu_1'])
        print ('Bleu_2:\t', final_scores['Bleu_2'])
        print ('Bleu_3:\t', final_scores['Bleu_3'])
        print ('Bleu_4:\t', final_scores['Bleu_4'])
        print ('METEOR:\t', final_scores['METEOR'])
        print ('ROUGE_L:', final_scores['ROUGE_L'])
        #print ('CIDEr:\t', final_scores['CIDEr'])
        #"""

        if get_scores:
            return final_scores
        
if __name__ == '__main__':
    cand = {'generated_description1': 'how are you', 'generated_description2': 'Hello how are you'}
    ref = {'generated_description1': ['what are you', 'where are you'],
           'generated_description2': ['Hello how are you', 'Hello how is your day']}
    x = Evaluate()
    x.evaluate(live=True, cand=cand, ref=ref)


import logging
from ..static import LOGGER_PRINT_LEVEL
logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(LOGGER_PRINT_LEVEL)

import numpy as np
import torch

class Predictor(object):
    def __init__(self, model, vectorizer, config):
        """
        Predictor class to evaluate for a given model.
        Args:
            model (seq2seq.models): trained model. This can be loaded from a checkpoint
                using `seq2seq.util.checkpoint.load`
            src_vocab (seq2seq.dataset.vocabulary.Vocabulary): source sequence vocabulary
            tgt_vocab (seq2seq.dataset.vocabulary.Vocabulary): target sequence vocabulary
        """
        self.config = config
        if self.config.cuda_is_available:
            self.model = model.cuda()
        else:
            self.model = model.cpu()
        self.model.eval()
        self.vectorizer = vectorizer

    def predict(self, src_seq, num_exams):
        """ Make prediction given `src_seq` as input.

        Args:
            src_seq (list): list of tokens in source language

        Returns:
            tgt_seq (list): list of tokens in target language as predicted
            by the pre-trained model
        """
        torch.set_grad_enabled(False)
        text = []
        for tok in src_seq:
            if tok in self.vectorizer.word2idx:
                text.append(self.vectorizer.word2idx[tok])
            else:
                text.append(3)

        input_variable = torch.LongTensor(text).view(1, -1)
        if self.config.cuda_is_available:
            input_variable = input_variable.cuda()

        input_lengths = [len(src_seq)]

        prev_generated_seq = None
        outputs = []
        for i in range(num_exams):
            _, _, other = \
                self.model(input_variable, prev_generated_seq, input_lengths)
            length = other['length'][0]

            tgt_id_seq = [other['sequence'][di][0].item() for di in range(length)]
            tgt_seq = [self.vectorizer.idx2word[tok] for tok in tgt_id_seq]
            output = ' '.join([i for i in tgt_seq if i != '<PAD>' and i != '<EOS>' and i != '<SOS>'])
            outputs.append(output)
            prev_generated_seq = torch.LongTensor(tgt_id_seq).view(1, -1)
            if self.config.cuda_is_available:
                prev_generated_seq = prev_generated_seq.cuda()
        return outputs

    def predict_batch(self, source, input_lengths, num_exams):
        torch.set_grad_enabled(False)
        output_seq = []
        input_variables = source
        for i in range(source.size(0)):
            title_id_seq = [input_variables[i][di].item() for di in range(input_lengths[i])]
            title_seq = [self.vectorizer.idx2word[tok] for tok in title_id_seq]
            title = ' '.join([k for k in title_seq if k != '<PAD>' and k != '<EOS>' and k != '<SOS>'])
            output_seq.append([title])
        prev_generated_seq = None
        for k in range(num_exams):
            _, _, other = \
                self.model(input_variables, prev_generated_seq, input_lengths)
            length = other['length']
            sequence = torch.stack(other['sequence'], 1).squeeze(2)
            prev_generated_seq = self._mask(sequence)
            for i in range(len(length)):
                opt_id_seq = [other['sequence'][di][i].item() for di in range(length[i])]
                opt_seq = [self.vectorizer.idx2word[tok] for tok in opt_id_seq]
                output = ' '.join([k for k in opt_seq if k != '<PAD>' and k != '<EOS>' and k != '<SOS>'])
                output_seq[i].append(output)
        return output_seq

    # Mask variable
    def _mask(self, prev_generated_seq):
        prev_mask = torch.eq(prev_generated_seq, 1).cpu().data.numpy()
        lengths = np.argmax(prev_mask,axis=1)
        max_len = prev_generated_seq.size(1)
        mask = []
        for i in range(prev_generated_seq.size(0)):
            if lengths[i] == 0:
                mask_line = [0] * max_len
            else:
                mask_line = [0] * lengths[i]
                mask_line.extend([1] * (max_len - lengths[i]))
            mask.append(mask_line)
        mask = torch.ByteTensor(mask)
        if self.config.cuda_is_available:
            mask = mask.cuda()
        return prev_generated_seq.data.masked_fill_(mask, 0)

    def preeval_batch(self, test_loader, abs_len, num_exams):
        torch.set_grad_enabled(False)
        refs = {}
        cands = []
        tmp = []
        for i in range(num_exams):
            cands.append({})
            tmp.append({})
        i = 0
        for batch_idx, (source, target, input_lengths) in enumerate(test_loader):
            input_variables = source
            input_lengths = input_lengths.tolist()
            prev_generated_seq = None
            for k in range(num_exams):
                _, _, other = \
                    self.model(input_variables, prev_generated_seq, input_lengths)
                length = other['length']
                sequence = torch.stack(other['sequence'], 1).squeeze(2)
                prev_generated_seq = self._mask(sequence)
                for j in range(len(length)):
                    out_seq = [other['sequence'][di][j] for di in range(length[j])]
                    out = self.prepare_for_bleu(out_seq)
                    tmp[k][j] = out
            for j in range(source.size(0)):
                i += 1
                ref = self.prepare_for_bleu(target[j])
                refs[i] = [ref]
                for k in range(num_exams):
                    cands[k][i] = tmp[k][j]
            if i % 100 == 0:
                print("Percentages:  %.4f" % (i/float(abs_len)))
        return cands, refs


    def prepare_for_bleu(self, sentence):
        sent=[x.item() for x in sentence if x.item() != 0 and x.item() != 1 and x.item() != 2]
        sent = ' '.join([str(x) for x in sent])
        return sent

    def predict_seq_title(self, title, sec_seq, num_exams):
        """ Make prediction given `src_seq` as input.

        Args:
            src_seq (list): list of tokens in source language

        Returns:
            tgt_seq (list): list of tokens in target language as predicted
            by the pre-trained model
        """
        torch.set_grad_enabled(False)
        text = []
        for tok in title:
            if tok in self.vectorizer.word2idx:
                text.append(self.vectorizer.word2idx[tok])
            else:
                text.append(3)

        input_variable = torch.LongTensor(text).view(1, -1)
        if self.config.cuda_is_available:
            input_variable = input_variable.cuda()

        input_lengths = [len(title)]

        text = []
        for tok in sec_seq:
            if tok in self.vectorizer.word2idx:
                text.append(self.vectorizer.word2idx[tok])
            else:
                text.append(3)

        prev_generated_seq = torch.LongTensor(text).view(1, -1)
        if self.config.cuda_is_available:
            prev_generated_seq = prev_generated_seq.cuda()

        outputs = []
        for i in range(num_exams):
            _, _, other = \
                self.model(input_variable, prev_generated_seq, input_lengths)
            length = other['length'][0]

            tgt_id_seq = [other['sequence'][di][0].item() for di in range(length)]
            tgt_seq = [self.vectorizer.idx2word[tok] for tok in tgt_id_seq]
            output = ' '.join([i for i in tgt_seq if i != '<PAD>' and i != '<EOS>' and i != '<SOS>'])
            outputs.append(output)
            prev_generated_seq = torch.LongTensor(tgt_id_seq).view(1, -1)
            if self.config.cuda_is_available:
                prev_generated_seq = prev_generated_seq.cuda()
        return outputs


# -*- coding: utf-8 -*-
# Python wrapper for METEOR implementation, by Xinlei Chen
# Acknowledge Michael Denkowski for the generous discussion and help

import os
import threading
import subprocess
import pkg_resources

from signal import signal, SIGPIPE, SIG_DFL 
signal(SIGPIPE,SIG_DFL)

METEOR_JAR = 'meteor-1.5.jar'
class Meteor(object):
    def __init__(self, language='en', norm=True):
        self.meteor_cmd = ['java', '-jar', '-Xmx2G', METEOR_JAR, '-', '-', '-stdio', '-l', language]

        if norm:
            self.meteor_cmd.append('-norm')

        self.meteor_p = subprocess.Popen(self.meteor_cmd, stdin=subprocess.PIPE,
            cwd=os.path.dirname(os.path.abspath(__file__)), 
                                        stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True, bufsize=1)
        # Used to guarantee thread safety
        self.lock = threading.Lock()

    def method(self):
        return "METEOR"

    def compute_score(self, gts, res):
        imgIds = sorted(list(gts.keys()))
        scores = []

        eval_line = 'EVAL'
        self.lock.acquire()
        for i in imgIds:
            assert(len(res[i]) == 1)

            hypothesis_str = res[i][0].replace('|||', '').replace('  ', ' ')
            score_line = ' ||| '.join(('SCORE', ' ||| '.join(gts[i]), hypothesis_str))

            # We obtained --> SCORE ||| reference 1 words ||| reference n words ||| hypothesis words
            self.meteor_p.stdin.write(score_line + '\n')
            stat = self.meteor_p.stdout.readline().strip()
            eval_line += ' ||| {}'.format(stat)

        # Send to METEOR
        self.meteor_p.stdin.write(eval_line + '\n')

        # Collect segment scores
        for i in range(len(imgIds)):
            score = float(self.meteor_p.stdout.readline().strip())
            scores.append(score)

        # Final score
        final_score = 100*float(self.meteor_p.stdout.readline().strip())
        self.lock.release()

        return final_score, scores

    def __del__(self):
        self.lock.acquire()
        self.meteor_p.stdin.close()
        self.meteor_p.wait()
        self.lock.release()
